#!/usr/bin/perl -T
# $Id: change_config.cgi,v 1.13 2019/01/23 07:27:44 sqko Exp $
use strict;
use warnings;
use ElchemeaHeader;
use ElchemeaUser;
use Elchemea;
use ElchemeaCGI qw(:DEFAULT :html :cgi);
use CGI qw(:standard :netscape);

print header();
my $u = param('user');
my $p = param('pass');
unless (&ElchemeaCGI::login_ok($u,$p))
{
    print "Could not change configuration value:\nAccess denied!\n";
    exit;
}
my $user = ElchemeaUser->new($u);
my $session = $user->session;
my $section = param('section');
my $key = param('key');
my $val = param('value');
unless (defined($section) && $section && defined($key) && $key)
{
    print "Error: Could not change configuration value!\n";
    $key = '' unless ($key);
    print "$key\n\n";
    exit;
}
my $warn = '';
my $pot = Elchemea::new_PSTAT(Elchemea::get_config_value('equipment','potentiostat'),Elchemea::get_config_value('equipment','potentiostat_address'));
my $fra = Elchemea::new_FRA(Elchemea::get_config_value('equipment','FRA'),Elchemea::get_config_value('equipment','FRA_address'));
if ($key =~ /delay|sleep/i && $val < 0)
{
    $val = 0;
    $warn = "$key can not be below 0!";
}
if ($key eq 'amplitude')
{
    $fra->set_mode($user->get_config_value($section,'mode')) if ($pot->virtual);
    if ($val > $fra->get_max_amplitude)
    {
	$val = $fra->get_max_amplitude;
	$warn = "Amplitude attempted outside valid range";
    }
    if ($val < $fra->get_min_amplitude)
    {
	$val = $fra->get_min_amplitude;
	$warn = "Amplitude attempted outside valid range";
    }
    if ($pot->type =~ /Solartron 128/i)
    {
	my $AC_gain = $user->get_config_value('impedance_potentiostat','use_0.01_factor');
	$AC_gain = $pot->get_default('use_0.01_factor') unless ($AC_gain);
	if ($AC_gain =~ /yes/i && $val > $fra->get_max_amplitude() / 100)
	{
	    $val = $fra->get_max_amplitude() / 100;
	    $warn = "Amplitude attempted outside valid range due to AC gain x 0.01 selected on potentiostat";
	}
    }
}
if ($key =~ /minimum_frequency|maximum_frequency/)
{
    if ($val < $fra->get_minfreqency)
    {
	$warn = 'Warning: frequency outside valid range attempted!';
	$val = $fra->get_minfreqency;
    }
    if ($val > $fra->get_maxfreqency)
    {
	$warn = 'Warning: frequency outside valid range attempted!';
	$val = $fra->get_maxfreqency;
    }
    if ($pot->get_minfreqency && $val < $pot->get_minfreqency)
    {
	$warn = 'Warning: frequency outside valid range attempted!';
	$val = $pot->get_minfreqency 
    }
    if ($pot->get_maxfreqency && $val > $pot->get_maxfreqency)
    {
	$warn = 'Warning: frequency outside valid range attempted!';
	$val = $pot->get_maxfreqency; 
    }    
}
if ($section eq 'impedance_FRA' && $key eq 'polarisation')
{
    $fra->set_mode($user->get_config_value($section,'mode')) if ($pot->virtual);
    if ($val < $fra->get_minpolarisation)
    {
	$warn = 'Warning: polarisation outside valid range attempted!';
	$val = $fra->get_minpolarisation;
    }
    if ($val > $fra->get_maxpolarisation)
    {
	$warn = 'Warning: polarisation outside valid range attempted!';
	$val = $fra->get_maxpolarisation 
    }
}
if (($section eq 'impedance_potentiostat'  || $section eq 'chrono' ) && $key eq 'polarisation')
{
    $pot->set_mode($user->get_config_value($section,'mode'));
    if ($val < $pot->get_minpolarisation)
    {
	$warn = 'Warning: polarisation outside valid range attempted!';
	$val = $pot->get_minpolarisation; 
    }
    if ($val > $pot->get_maxpolarisation)
    {
	$warn = 'Warning: polarisation outside valid range attempted!';
	$val = $pot->get_maxpolarisation;
    }
}
if ($section eq 'potsweep'  && $key eq 'sweep_rate')
{
    if ($val < $pot->get_minsweeprate)
    {
	$warn = 'Warning: sweep rate outside valid range attempted!';
	$val = $pot->get_minsweeprate; 
    }
    if ($val > $pot->get_maxsweeprate)
    {
	$warn = 'Warning: sweep rate outside valid range attempted!';
	$val = $pot->get_maxsweeprate;
    }
}
my $res = $user->change_config_value($section,$key,$val);
my @list = $user->get_impedance_frequencies($user->session,$pot);
my $ext_time = 0;
if ($section eq 'chrono' || $section eq 'potsweep' || $section eq 'impedance_FRA' || $section eq 'impedance_potentiostat')
{
    my $mode = $section;
    $mode = 'impedance' if ($section =~ /impedance/);
    $ext_time = $user->get_offset_program_time($mode);
}
my $time = Elchemea::format_time($fra->get_impedance_time(\@list) + $ext_time);
if ($res)
{
    if ($warn)
    {
	print "WARN\n";
    }
    else
    {
	print "OK\n";
    }
    print "$key\n";
    print "$val\n";
    print "$time\n";
    print "$warn\n";
}
else
{
    print "Error: Could not change configuration value!\n";
    print "$key\n\n";
}
exit;
