#!/usr/bin/perl -w
#$Id: elchemea.cgi,v 1.2 2018/10/26 09:40:46 sqko Exp $
use strict;
use warnings;
use Data::Dumper;
use SOAP::Transport::HTTP;

my $server = SOAP::Transport::HTTP::CGI
    -> dispatch_to('Elchemea')
    -> handle;

package Elchemea;
use vars qw(@ISA);
@ISA = qw(SOAP::Server::Parameters);
use ElchemeaHeader;
use Elchemea;
use ElchemeaUser;
use ElchemeaProgram;
use Sys::Hostname;
use IO::Interface::Simple;

=begin WSDL
_DOC Returns the hostname of the server
_RETURN $string
=end WSDL

=cut

sub host()
{
    my $class = shift;
    return SOAP::Data->new(name=>'result',value=>hostname(),type=>'string');
}


=begin WSDL
_DOC Returns the hostname of the server
_RETURN $string
=end WSDL

=cut

sub ping()
{
    my $class = shift;
    my $host = hostname();
    #### Try to get ip address of netwrok interface ###
    my @ilst = ('eth','em','en','lo');
    my $local = '';
    my @ifl = IO::Interface::Simple->interfaces();
  OUTER:
    foreach my $try (@ilst)
    {
	foreach my $interface (@ifl)
	{
	    if ($interface =~ /$try/)
	    {
		my $if = IO::Interface::Simple->new($interface);
		$local = $if->address();
		last OUTER if ($local);
	    }
	}
    }
    ### Use localhost if everything else fails ###
    $local = 'localhost' unless ($local);
    return SOAP::Data->new(name=>'result',value=>"Elchemea system on $host on addr:$local listening on http",type=>'string');
}

=begin WSDL
_DOC Returns the setup mode of the server
_RETURN $string
=end WSDL

=cut


sub mode
{
    my $class = shift;
    my $compat = get_config_value('global','compatibility');
    if ($compat && $compat =~ /yes/i)
    {
	return SOAP::Data->new(name=>'result',value=>'1260',type=>'string');
    }
    else
    {
        my $fra = Elchemea::new_FRA(Elchemea::get_config_value('equipment','FRA'),Elchemea::get_config_value('equipment','FRA_address'));
        my $pot = Elchemea::new_PSTAT(Elchemea::get_config_value('equipment','potentiostat'),Elchemea::get_config_value('equipment','potentiostat_address'));
        my $str = $pot->type.':'.$fra->type;
        $str =~ s/\s+/_/g;
	return SOAP::Data->new(name=>'result',value=>$str,type=>'string');
    }
}

=begin WSDL
_DOC Returns the session number for the specified user
_IN username $string Username
_RETURN $int
=end WSDL

=cut



sub session
{
    my $class = shift;
    my $rid = shift;
    my $user = ElchemeaUser->new($rid);
    my $session = $user->session();
    return SOAP::Data->new(name=>'result',value=>$session,type=>'integer');
}

=begin WSDL
_DOC Starts an impedance scan for the specified user using the mode and session number specified. Note that the mode must be specified althouth it is not used!
_IN username $string Username
_IN mode $string Control mode, not used but must be specified
_IN session $int Session number from which to use configuration
_RETURN $string
=end WSDL

=cut


sub impedance
{
    my $class = shift;
    my $rid = shift;
    my $user = ElchemeaUser->new($rid);
    my $mode = shift;
    my $session = shift;
    my $dot_addr = $ENV{REMOTE_ADDR};
    $user->proglog("Impedance started remotely from $dot_addr");
    my $res = Elchemea::CGI_client('impedance',$user->name,$session);
    return SOAP::Data->new(name=>'result',value=>$res,type=>'string');
}

=begin WSDL
_DOC Starts an chrono amperometry/potentiometry measruement for the specified user using the mode and session number specified. Note that the mode must be specified althouth it is not used!
_IN username $string Username
_IN mode $string Control mode, not used but must be specified
_IN session $int Session number from which to use configuration
_RETURN $string
=end WSDL

=cut



sub chrono
{
    my $class = shift;
    my $rid = shift;
    my $user = ElchemeaUser->new($rid);
    my $mode = shift;
    my $session = shift;
    my $dot_addr = $ENV{REMOTE_ADDR};
    $user->proglog("Chrono started remotely from $dot_addr");
    my $str = Elchemea::CGI_client('chrono',$user->name,$session);
    return SOAP::Data->new(name=>'result',value=>$str,type=>'string');
}



=begin WSDL
_DOC Starts a potential sweep for the specified user using the mode and session number specified. Note that the mode must be specified althouth it is not used!
_IN username $string Username
_IN mode $string Control mode, not used but must be specified
_IN session $int Session number from which to use configuration
_RETURN $string
=end WSDL

=cut


sub potsweep
{
    my $class = shift;
    my $rid = shift;
    my $user = ElchemeaUser->new($rid);
    my $mode = shift;
    my $session = shift;
    my $dot_addr = $ENV{REMOTE_ADDR};
    $user->proglog("Potsweep started remotely from $dot_addr");
    my $res = Elchemea::CGI_client('potsweep',$user->name,$session);
    return SOAP::Data->new(name=>'result',value=>$res,type=>'string');
}


=begin WSDL
_DOC Changes the multiplexer channel to the value specified
_IN username $string Username
_IN mode $string Control mode, not used but must be specified
_IN channel $int Multiplexer channel to select
_RETURN $string
=end WSDL

=cut

sub change_channel
{
    my $class = shift;
    my $rid = shift;
    my $id = ElchemeaProgram->new($rid);
    my $user = ElchemeaUser->new($rid);
    my $mode = shift; ### Dummy ###
    my $chid = shift;
    my $dot_addr = $ENV{REMOTE_ADDR};
    $id->change_channel($chid);
    $user->proglog("Changed multiplexer channel to $chid remotely from $dot_addr");
    return SOAP::Data->new(name=>'result',value=>"Changed multiplexer channel to $chid",type=>'string');
}


=begin WSDL
_DOC Resets potentiosta/FRA devices
_IN username $string Username
_RETURN $string
=end WSDL

=cut

sub reset_potentiostat
{
    my $class = shift;
    my $rid = shift;
    my $id = ElchemeaProgram->new($rid);
    my $user = ElchemeaUser->new($rid);
    my $dot_addr = $ENV{REMOTE_ADDR};
    $id->reset_potentiostat;
    $user->proglog("Potentiostat reset remotely from $dot_addr");
    return SOAP::Data->new(name=>'result',value=>"Potentiostat reset",type=>'string');
}

=begin WSDL
_DOC Returns true if the system is performing a measurement
_RETURN $boolean
=end WSDL

=cut


sub is_measuring
{
    my $class = shift;
    my $res = 0;
    $res = 1 if (Elchemea::is_locked);
    return SOAP::Data->new(name=>'result',value=>$res,type=>'boolean');
}

=begin WSDL
_DOC Returns true if a user program is running
_RETURN $boolean
=end WSDL

=cut


sub is_program_running
{
    my $class = shift;
    my $res = 0;
    $res = 1 if (-e $programlock);
    return SOAP::Data->new(name=>'result',value=>$res,type=>'boolean');
}

=begin WSDL
_DOC Gets the content of the specified file id for the specified user, mode and and session. If a measurement is running (either for this file or an other) the function simply returns 'MEASURING'
_IN username $string Username
_IN mode $string Control mode, not used but must be specified
_IN session $int Session number
_IN fileid $int ID (number) of the file which needs to be retrieved 
_RETURN $string
=end WSDL

=cut


sub get_file
{
    my $class = shift;
    return SOAP::Data->new(name=>'result',value=>"MEASURING",type=>'string') if (Elchemea::is_locked);
    my $user = ElchemeaUser->new(shift);
    my $mode = shift;
    my $session = shift;
    my $fileid = shift;
    $fileid = sprintf("%04d",$fileid) if ($fileid < 1000);
    my $file = $userdir.$user->name.'/'.$session.'/s'.$session.'_'.$fileid.'.';
    my $filename = '';
    foreach ('i2b','i2c','i2p')
    {
        my $f = $file.$_;
        next unless -e $f;
        $filename = $file. $_;
        last;
    }
    return SOAP::Data->new(name=>'result',value=>'',type=>'string') unless $filename;
    my $id = SemaforeFile->new($filename);
    my $string = join("\n",$id->readlines);
    return SOAP::Data->new(name=>'result',value=>$string,type=>'string');
}

=begin WSDL
_DOC Gets the content of the specified file id for the specified user, mode and and session. As opposed to get_file, this function returns the content even if the system is still measuring.
_IN username $string Username
_IN mode $string Control mode, not used but must be specified
_IN session $int Session number
_IN fileid $int ID (number) of the file which needs to be retrieved 
_RETURN $string
=end WSDL

=cut

sub get_file_now
{
    my $class = shift;
    my $user = ElchemeaUser->new(shift);
    my $mode = shift;
    my $session = shift;
    my $fileid = shift;
    $fileid = sprintf("%04d",$fileid) if ($fileid < 1000);
    my $file = $userdir.$user->name.'/'.$session.'/s'.$session.'_'.$fileid.'.';
    my $filename = '';
    foreach ('i2b','i2c','i2p')
    {
        my $f = $file.$_;
        next unless -e $f;
        $filename = $file. $_;
        last;
    }
    return SOAP::Data->new(name=>'result',value=>'',type=>'string') unless $filename;
    my $id = SemaforeFile->new($filename);
    my $string = join("\n",$id->readlines);
    return SOAP::Data->new(name=>'result',value=>$string,type=>'string');
}


